//
//  ViewController.swift
//  TableConcept
//
//  Created by Dimitar Shopovski on 2/8/21.
//

import UIKit

class ViewController: UIViewController, UITableViewDelegate, UITableViewDataSource {

    let batchSize: Int = 30
    let totalElements: Int = 480
    var startPage: Int = 5
    var elements: [Int?] = []
    var isNewDataLoading: Bool = false
    var loadedPages = [Int]()
    
    @IBOutlet weak var mainTableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initialLoad()
        mainTableView.prefetchDataSource = self
    }
    
    func initialLoad() {
                
        for _ in 0..<totalElements {
            elements.append(.none)
        }
        
        loadedPages.append(startPage)
        
        for value in startPage*batchSize..<(startPage+1)*batchSize {
            elements[value] = value
        }
        
        mainTableView.reloadData()
        let toIndex = startPage*batchSize + ((startPage+1)*batchSize - startPage*batchSize)/2
        mainTableView.scrollToRow(at: IndexPath(row: toIndex, section: 0), at: .middle, animated: true)
    }
    
    func getNewData(page: Int) {
                        
        if isNewDataLoading {
            return
        }
        
        isNewDataLoading = true
        
        var temp: [Int] = [Int]()
        for num in page*batchSize..<(page+1)*batchSize {
            elements[num] = num
            temp.append(num)
        }
        
        loadedPages.append(page)
        
        let indexes: [IndexPath] = temp.map {
            return IndexPath(row: $0, section: 0)
        }
        
        mainTableView.reloadRows(at: indexes, with: .none)
        isNewDataLoading = false
    }
    
    //MARK: - TableView methods
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return elements.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "Cell", for: indexPath) as! TestTableViewCell
        if isLoadingCell(for: indexPath) {
            cell.configure(with: .none)
        } else {
            cell.configure(with: elements[indexPath.row])
        }
        return cell
    }
    
    func isLoadingCell(for indexPath: IndexPath) -> Bool {
        return elements[indexPath.row] == .none
    }
    
    func visibleIndexPathsToReload(intersecting indexPaths: [IndexPath]) -> [IndexPath] {
        let indexPathsForVisibleRows = mainTableView.indexPathsForVisibleRows ?? []
        let indexPathsIntersection = Set(indexPathsForVisibleRows).intersection(indexPaths)
        return Array(indexPathsIntersection)
    }
}

extension ViewController: UITableViewDataSourcePrefetching {
    func tableView(_ tableView: UITableView, prefetchRowsAt indexPaths: [IndexPath]) {
        if indexPaths.contains(where: isLoadingCell) {
            
            let index = indexPaths[0]
            let page = index.row/batchSize
            if !loadedPages.contains(page) {
                //fetch new
                DispatchQueue.main.asyncAfter(deadline: DispatchTime.now() + 2.0) {
                    self.getNewData(page: page)
                }
            }
        }
    }
}
